from getpass import getpass
import sys

from sf2.args import get_args
from sf2.cipher import Cipher

try:
    from sf2.editor import Editor
except Exception as e:
    def Editor(*args, **kwargs):
        print(f"No editor support ! ({e})")
        sys.exit(-1)



def main():
    args = get_args()

    if args.encrypt:
        print("We recommand min 12 chars with a-z, A-Z, 0-9 and special symbol")
        password1 = getpass("Password : ")
        password2 = getpass("Confirm password : ")
        if password1 != password2:
            print("Password are not the same, abord")
            sys.exit(-1)
        password = password1
        cipher = Cipher()
        try:
            cipher.encrypt_file(password, args.infilename, args.outfilename)
        except Exception as e:
            print(f"Failed to encrypt file {args.infilename} : {e}")
            sys.exit(-1)

    elif args.decrypt:
        password = getpass()
        cipher = Cipher()
        try:
            cipher.decrypt_file(password, args.infilename, args.outfilename)
        except Exception as e:
            print(f"Failed to decrypt file {args.infilename} : {e}")
            sys.exit(-1)

    elif args.verify:
        password = getpass()
        cipher = Cipher()
        
        try:
            is_ok = cipher.verify_file(password, args.infilename)
        except Exception as e:
            print(f"Failed to verify file {args.infilename} : {e}")
            sys.exit(-1)

        if is_ok :
            print("OK")
        else:
            print("FAILED !")
            sys.exit(-1)

    if args.new:
        print("We recommand min 12 chars with a-z, A-Z, 0-9 and special symbol")
        password1 = getpass("Password : ")
        password2 = getpass("Confirm password : ")
        if password1 != password2:
            print("Password are not the same, abord")
            sys.exit(-1)
        password = password1
        cipher = Cipher()
        try:
            encrypted = cipher.encrypt(password, "") 

            with open(args.infilename, "w") as f:
                f.write(encrypted)

        except Exception as e:
            print(f"Failed to encrypt file {args.infilename} : {e}")
            sys.exit(-1)

    elif args.edit:
        password = getpass()
        cipher = Cipher()

        try:
            is_ok = cipher.verify_file(password, args.infilename)
        except Exception as e:
            print(f"Failed to open file {args.infilename} : {e}")
            sys.exit(-1)

        if not is_ok :
            print(f"Failed to open file {args.infilename} : bad key ?")
            sys.exit(-1)

        editor = Editor(password, args.infilename)
        editor.load()
        editor.create()

    sys.exit(0)        

if __name__ == "__main__":
    main()


import base64
from operator import contains
import os
import json

from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

class Cipher:
    def __init__(self) -> None:
        pass

    def password_2_key(self, salt:bytes, password:str)->bytes:
        """
        It takes a salt and a password and returns a key for Fernet module
        
        :param salt: a random string of bytes
        :type salt: bytes
        :param password: The password to use for the key derivation
        :type password: str
        :return: The key is being returned.
        """
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=salt,
            iterations=480000,
        )
        password = bytes(password, "utf8")
        key = base64.urlsafe_b64encode(kdf.derive(password))

        return key

    def create_salt(self, _rand=os.urandom)->bytes:
        """
        It creates a random 16 byte string, and then encodes it using base64
        
        :param _rand: This is a function that returns a random byte string. The default is os.urandom,
        which is a secure random number generator
        :return: A random string of bytes that is encoded in base64.
        """
        salt = _rand(16)
        return base64.urlsafe_b64encode(salt)

    def encrypt(self, password:str, data:str)->dict:
        """
        > It takes a password and some data, creates a salt, creates a key from the salt and password,
        encrypts the data with the key, and returns the salt and encrypted data in a JSON object.
        
        :param password: The password that will be used to encrypt the data
        :type password: str
        :param data: The data to be encrypted
        :type data: str
        :return: A JSON string containing the salt and the encrypted data.
        """
        salt = self.create_salt()
        key = self.password_2_key(salt, password)
        
        f = Fernet(key)
        encrypted = f.encrypt(bytes(data, "utf8"))

        container = {
            "salt" : str(salt, "utf8"),
            "data" : str(encrypted, "utf8")
        }
        return json.dumps(container, indent=4)


    def decrypt(self, password:str, container_bytes:bytes)->str:
        """
        > It takes a password and a container, and returns the plaintext
        
        :param password: The password you want to use to encrypt the data
        :type password: str
        :param container_bytes: the bytes of the container file
        :type container_bytes: bytes
        :return: The decrypted data.
        """
        container = json.loads(container_bytes)
        data = bytes(container["data"], "utf8")
        salt = bytes(container["salt"], "utf8")

        key = self.password_2_key(salt, password)
        f = Fernet(key)
        plain = f.decrypt(data)

        return str(plain, "utf8")

    def encrypt_file(self, password:str, infilename:str, outfilename:str)->None:
        """
        It opens the file, reads the contents, encrypts the contents, and writes the encrypted contents
        to a file
        
        :param password: The password to use to encrypt the file
        :type password: str
        :param infilename: The name of the file to encrypt
        :type infilename: str
        :param outfilename: The name of the file to write the encrypted data to. If this is None, the
        encrypted data will be printed to the console
        :type outfilename: str
        """
        with open(infilename, "r") as f:
            plain = f.read()

        container = self.encrypt(password, plain)

        if outfilename is None:
            print(container)
        else:
            with open(outfilename, "w") as f:
                f.write(container)

    def decrypt_file(self, password:str, infilename:str, outfilename:str)->None:
        """
        It decrypts a file.
        
        :param password: The password to use for encryption/decryption
        :type password: str
        :param infilename: The name of the file to be decrypted
        :type infilename: str
        :param outfilename: The name of the file to write the decrypted text to. If this is None, then
        the decrypted text is printed to the console
        :type outfilename: str
        """
        with open(infilename, "r") as f:
            container = f.read()

        plain = self.decrypt(password, container)

        if outfilename is None:
            print(plain)
        else:
            with open(outfilename, "w") as f:
                f.write(plain)

    def verify_file(self, password:str, infilename:str)->bool:
        """
        > It opens the file, reads the contents, and then tries to decrypt it. If it can decrypt it, it
        returns True. If it can't decrypt it, it returns False
        
        :param password: The password to use for encryption/decryption
        :type password: str
        :param infilename: the name of the file to be encrypted
        :type infilename: str
        :return: a boolean value.
        """
        
        with open(infilename, "r") as f:
            container = f.read()
        try:
            plain = self.decrypt(password, container)
            return True
        except:
            return False






import argparse
from argparse import RawTextHelpFormatter

DESCRIPTION = """

   _____ _________ 
  / ___// ____/__ \\
  \__ \/ /_   __/ /
 ___/ / __/  / __/ 
/____/_/    /____/ 
                   
Simple Fernet File

This tool manages an encrypted file with the Fernet algorithm.

By Laulin
Supported by Spartan Conseil
"""

def get_args():
    parser = argparse.ArgumentParser(description=DESCRIPTION, formatter_class=RawTextHelpFormatter)
    action = parser.add_mutually_exclusive_group(required=True)
    action.add_argument('-e', action='store_true', default=False, dest='encrypt', help='Encrypt a file')
    action.add_argument('-d', action='store_true', default=False, dest='decrypt', help='Decrypt a file. If -o is not provided, stdout is used')
    action.add_argument('-v', action='store_true', default=False, dest='verify', help='Check if the key is valid on the file')
    action.add_argument('-n', action='store_true', default=False, dest='new', help='Create an empty encrypted file')
    action.add_argument('--edit', action='store_true', default=False, dest='edit', help='Run the internal editor [not implemented]')

    parser.add_argument('-i', default=None, dest='infilename', help='Select the encrypt file pass', required=True)
    parser.add_argument('-o', default=None, dest='outfilename', help='Select the encrypt file pass')
    return parser.parse_args()



import dearpygui.dearpygui as dpg

from sf2.cipher import Cipher

KEY_CTRL = 341
KEY_C = 83
KEY_ALL = -1

class Editor:
    def __init__(self, password:str, filename:str) -> None:
        self._password = password
        self._filename = filename

        self._last_saved = None
        self._data = None

    def load(self):
        with open(self._filename) as f:
            container = f.read()
        cipher = Cipher()
        self._data = cipher.decrypt(self._password, container)
        self._last_saved = self._data

    def _create_menu(self)->None:
        with dpg.viewport_menu_bar():
            with dpg.menu(label="File"):
                dpg.add_menu_item(label="Save (ctrl+s)", callback=self.save)
                dpg.add_menu_item(label="Quit", callback=self.on_close)

    def create(self):
        dpg.create_context()
        dpg.configure_app(init_file=self._filename+".ini")

        with dpg.window(tag=self._filename, on_close=self.on_close, width=600, height=400):
            self._create_menu()
            dpg.add_input_text(default_value=self._data, multiline=True, readonly=False, tag="text", pos=(5,20), width=-1, height=-1, tab_input=True)
            

            with dpg.handler_registry():
                dpg.add_key_press_handler(key=KEY_CTRL, callback=self.on_key_ctrl)

        with dpg.window(label="Save changes ?", modal=True, show=True, tag="modal_id", no_title_bar=True):
            dpg.add_text("Your document's changed is not save\nDo you want to save ?")
            dpg.add_separator()
            with dpg.group(horizontal=True):
                dpg.add_button(label="Yes", width=75, callback=self.save_and_close)
                dpg.add_button(label="No", width=75, callback=self.save_and_discard)

        dpg.create_viewport(title='SF2', width=600, height=400,)
        dpg.setup_dearpygui()
        dpg.show_viewport()
        dpg.set_exit_callback(self.on_exit)
        dpg.start_dearpygui()
        dpg.destroy_context()

    def save(self):
        self._data = dpg.get_value("text")
        cipher = Cipher()
        container = cipher.encrypt(self._password, self._data)
        with open(self._filename, "w") as f:
            f.write(container)

        self._last_saved = self._data

    def on_key_ctrl(self, sender, app_data):
        if dpg.is_key_down(KEY_C):
            self.save()

    def on_close(self):
        self._data = dpg.get_value("text")
        if self._data != self._last_saved:
            dpg.configure_item("modal_id", show=True)
        else:
            dpg.stop_dearpygui()

    def save_and_close(self):
        self.save()
        dpg.stop_dearpygui()

    def save_and_discard(self):
        dpg.stop_dearpygui()

    def on_exit(self):
        dpg.save_init_file(self._filename+".ini")


from setuptools import setup, find_packages
 
setup(name='sf2',
    version='1.0.0',
    url='',
    license='Apachev2',
    author='Laurent MOULIN',
    author_email='gignops@gmail.com',
    description='Encrypt and decrypt your file with Fernet algorithm',
    packages=find_packages(exclude=['tests', "etc", "build", "dist", "sf2.egg-info"]),
    install_requires=["cryptography", "dearpygui"],
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    zip_safe=False,
    classifiers=[
      "Programming Language :: Python :: 3",
      "License :: OSI Approved :: Apache Software License",
      "Operating System :: OS Independent",
      "Development Status :: 5 - Production/Stable",
      "Intended Audience :: Information Technology",
      "Topic :: Security :: Cryptography"
    ],
    python_requires='>=3',
      entry_points={
            'console_scripts': [ 
            'sf2 = sf2.sf2:main' 
            ] 
      }
)